'''
# @Author       : Chr_
# @Date         : 2020-07-30 17:50:27
# @LastEditors  : Chr_
# @LastEditTime : 2020-08-21 12:23:09
# @Description  : 网络模块,负责网络请求
'''

import time
import random
import logging
import traceback
from requests import Session, Response
from json import JSONDecodeError
from urllib.parse import urlparse

from .static import HEYBOX_VERSION, Android_UA, iOS_UA, URLS
from .static import ENC_STATIC, CommentType, ReportType
from .utils import md5_calc, encrypt_data, b64encode, rsa_encrypt, gen_random_str
from .error import ClientException, Ignore, UnknownError, TokenError


class Network():
    __session = Session()
    __session.headers = {}
    __headers = {}
    __cookies = {}
    __params = {}
    __heybox_id = 0
    __sleep_interval = 1.0
    __auto_report = True

    logger = logging.getLogger('-')

    def __init__(self, account: dict, hbxcfg: dict, debug: bool):
        super().__init__()

        try:
            os_type = hbxcfg.get('os_type', 1)
            os_version = hbxcfg.get('os_version', '9')
            channel = hbxcfg.get('channel', 'heybox_yingyongbao')
            sleep_interval = hbxcfg.get('sleep_interval', 1)
            auto_report = hbxcfg.get('auto_report', True)
            heybox_id = account.get('heybox_id', -1)
            imei = account.get('imei', gen_random_str(16))
            pkey = account.get('pkey', None)
        except (AttributeError, KeyError):
            raise ClientException('传入参数类型错误')

        self.__headers = {'Referer': 'http://api.maxjia.com/',
                          'User-Agent': Android_UA if os_type == 1 else (iOS_UA % os_version),
                          'Host': 'api.xiaoheihe.cn', 'Connection': 'Keep-Alive',
                          'Accept-Encoding': 'gzip'}
        if pkey:
            self.__cookies = {'pkey': pkey}
        self.__params = {'heybox_id': heybox_id, 'imei': imei,
                         'os_type': 'Android' if os_type == 1 else 'iOS',
                         'os_version': os_version, 'version': HEYBOX_VERSION, '_time': '',
                         'hkey': '', 'channel': channel}
        if os_type == 2:  # 模拟IOS客户端
            self.__params.pop('channel')

        log_level = 10 if debug else 20
        log_format = '%(asctime)s [%(levelname)s][%(name)s]%(message)s'
        log_time = '%H:%M:%S'
        logging.basicConfig(level=log_level,
                            format=log_format,
                            datefmt=log_time)

        self.__heybox_id = heybox_id
        self.__sleep_interval = sleep_interval
        self.__auto_report = auto_report
        self.logger = logging.getLogger(str(heybox_id))
        self.logger.debug('网络模块初始化完毕')

    def debug(self):
        '''
        仅供调试
        '''
        pass

    @property
    def heybox_id(self):
        '''
        获取heybox_id
        '''
        return(self.__heybox_id)

    def data_report(self, rtype: int, value: tuple = None) -> bool:
        '''
        模拟客户端汇报数据

        参数:
            rtype: 汇报模板,参见status.ReportType
            value: 根据模板不同有不同的作用
        返回:
            bool: 操作是否成功
        '''
        if rtype == ReportType.Source:
            # 并未完成
            return(False)
            p = {'type': 13}
            d = {"source": [{'pageID': '1'}, {'pageID': '12'}]}
        elif rtype == ReportType.Quit:
            # 并未完成
            return(False)
            p1 = {'type': 99}
            d1 = {"events": [{"event_id": "203", "time": str(
                int(time.time())), "type": "show"}]}
            p2 = {'type': 100}
            d2 = {"events": [{"event_id": "176", "time": str(
                int(time.time())), "value": 3096}]}
        elif rtype == ReportType.View:
            p = {'type':	9, 'viewTime'	: random.randint(1, 20),
                 'index': value[0], 'h_src': b64encode('news_feeds_-1')	,
                 'link_id':	value[1]}

        url = URLS.DATA_REPORT
        try:
            if rtype == ReportType.Source:
                self.__post_encrypt_without_report(url=url, data=d, params=p)
            elif rtype == ReportType.Quit:
                self.__post_encrypt_without_report(url=url, data=d1, params=p1)
                self.__post_encrypt_without_report(url=url, data=d2, params=p2)
            else:
                self.__get_without_report(url=url, params=p)
            self.logger.debug('模拟客户端上报数据成功')
            return(True)
        except ClientException as e:
            self.logger.error(f'上报数据出错 [{e}]')
            return(False)

    def random_sleep(self, min_t: int, max_t: int):
        '''
        随机延时

        参数:
            min_t: 最短时间
            max_t: 最长时间
        '''
        x = self.__sleep_interval
        if x:
            t = random.randint(min_t, max_t)
            t *= x
            t += random.random()
            self.logger.debug(f'随机延时 {"%.2f" % t} 秒')
            time.sleep(t)
        else:
            self.logger.debug('随机延时已禁用')

    def _login(self, phone: int, password: str) -> (int, str, str):
        '''
        使用手机号密码登陆小黑盒,登陆失败返回False
        会把heybox_id设为-1

        参数:
            phone: 手机号
            password: 密码
        返回:
            int: heybox_id
            str: imei
            str: pkey
        '''
        url = URLS.LOGIN
        # 清除登陆凭据信息, 生成随机imei
        self.__cookies = {}
        self.__heybox_id = -1
        imei = gen_random_str(16)
        self.__params['imei'] = imei

        phone_d = rsa_encrypt(f'+86{phone}')
        password_d = rsa_encrypt(password)
        data = {'phone_num': phone_d, 'pwd': password_d}
        try:
            result = self._post(url=url, data=data)
            ad = result['account_detail']
            name = ad['username']
            heybox_id = int(ad['userid'])
            pkey = result['pkey']
            self.logger.debug(f'登录成功,[{name}]@{heybox_id}')
            # 设置登陆凭据
            self.__cookies = {'pkey': pkey}
            self.__heybox_id = heybox_id
            return((heybox_id, imei, pkey))

        except (TokenError, ValueError) as e:
            self.logger.error(f'登录失败 [{e}]')
            return(False)

    def __flash_token(self, url: str) -> int:
        '''
        根据当前时间生成time_和hkey,并存入self._parames中

        参数:
            url: url路径
        返回:
            int: 整数时间戳
        '''
        def url_to_path(url: str) -> str:
            path = urlparse(url).path
            if path and path[-1] == '/':
                path = path[:-1]
            return(path)

        def encode(url: str, t: int) -> str:
            enc = list(f'{url}{ENC_STATIC}{t}')
            count = len(enc) - 1
            for i in range(0, count):
                for j in range(0, count-i):
                    if (enc[j] > enc[j+1]):
                        enc[j], enc[j+1] = enc[j+1], enc[j]
            l = len(enc) // 3 + 1
            enc += '\0'
            enc = [enc[3*i] for i in range(0, l)]
            if enc[-1] == '\0':
                enc = enc[:-1]
            enc += list(hex(t))
            count = len(enc)-1
            for i in range(0, count):
                for j in range(0, count-i):
                    if (enc[j] > enc[j+1]):
                        enc[j], enc[j+1] = enc[j+1], enc[j]
            result = ''.join(enc)
            return(result)

        t = int(time.time())
        u = url_to_path(url)
        h = encode(u, t)
        h = h.replace('x', '')
        h = md5_calc(h)
        h = h[:10]
        p = self.__params
        p['_time'] = t
        p['hkey'] = h
        return(t)

    def __check_status(self, jd: dict):
        '''
        检查json字典,检测到问题抛出异常

        参数:
            jd:json字典
        '''
        try:
            status = jd['status']
            if status == 'ok':
                return (True)
            elif status == 'ignore':
                raise Ignore
            elif status == 'failed':
                msg = jd['msg']
                if msg in ('操作已经完成', '不能进行重复的操作哦',
                           '不能重复赞哦', '不能给自己的评价点赞哟',
                           '自己不能粉自己哦', '您已经加入了房间',
                           ''):
                    raise Ignore

                elif msg in ('抱歉,没有找到你要的帖子',
                             '操作失败', 'error link_id',
                             '错误的帖子', '错误的用户',
                             '该帖已被删除', '帖子已被删除',
                             '加入房间失败，已到开奖时间',
                             '该用户已注销', '数据上报失败'):
                    raise ClientException(f'客户端出错@{msg}')

                elif msg in ('用户名格式错误', '用户名不存在'
                             '密码格式错误',
                             '用户名或密码错误或者登录过于频繁',
                             '你的账号已被限制访问，如有疑问请于管理员联系'):
                    raise TokenError(f'登录失败@{msg}')

                elif msg == '系统时间不正确':
                    raise OSError('系统时间错误')

                elif msg == '出现了一些问题，请稍后再试':
                    self.logger.error(f'返回值:{jd}')
                    self.logger.error('出现这个错误的原因未知,有可能是访问频率过快,请过一会再重新运行脚本')
                    raise UnknownError(f'返回值:{jd}')

                elif msg == '参数错误':
                    self.logger.error(f'返回值:{jd}')
                    self.logger.error('请求参数错误, 有可能是小黑盒更新了, 请更新脚本')
                    raise UnknownError(f'返回值:{jd}')

                self.logger.error(f'未知的返回值[{msg}]')
                self.logger.error('请将以下内容发送到chr@chrxw.com')
                self.logger.error(f'{jd}')
                self.logger.error(f'{traceback.print_stack()}')
                raise UnknownError(f'未知的返回值[{msg}]')
            elif status == 'relogin':
                raise TokenError('账号凭据过期,请重新登录')
        except (KeyError, ValueError, NameError, AttributeError):
            self.logger.debug(f'JSON格式错误')
            self.logger.debug(f'{jd}')
            self.logger.error(f'{traceback.format_exc()}')
            raise UnknownError(f'未知的返回值[{jd}]')

    def __get_json(self, resp: Response) -> dict:
        '''
        把Response对象转成json字典,出错返回{}

        参数:
            resp: Response对象
        返回:
            dict: json字典
        '''
        try:
            jd = resp.json()
            self.logger.debug(f'返回值 [{jd}]')
            self.__check_status(jd)
            return(jd)
        except JSONDecodeError:
            self.logger.warning(f'[*] JSON解析失败 [{resp.text}]')
            return({})

    def _get(self, url: str, params: dict = None,  headers: dict = None,
             cookies: dict = None, key: str = 'result') -> dict:
        '''
        GET方法发送请求

        参数:
            url: URL
            [params]: 请求参数,会添加到self._params前面
            [headers]: 请求头,会替换self._headers
            [cookies]: 请求头,会替换self._cookies
            [key]: 要返回的数据键名,默认为'result',留空表示返回原始json
        返回:
            dict: json字典
        '''
        result = self.__get_without_report(
            url=url,
            params=params,
            headers=headers,
            cookies=cookies,
            key=key)

        if self.__auto_report and params:
            i = params.get('index')
            l = params.get('link_id')
            if i and l and random.random() <= 0.7:  # 70%概率触发
                self.data_report(ReportType.View, (i, l))

        return(result)

    def _post(self, url: str, params: dict = None, data: dict = None, headers: dict = None,
              cookies: dict = None, key: str = 'result') -> dict:
        '''
        POST方法发送请求

        参数:
            url: URL
            [params]: 请求参数,会添加到self._params前面
            [data]: 请求体
            [headers]: 请求头,会替换self._headers
        返回:
            Response: 请求结果
        '''
        self.__flash_token(url)
        p = {**(params or {}), **self.__params}
        d = data or {}
        h = headers or self.__headers
        c = cookies or self.__cookies
        resp = self.__session.post(
            url=url, params=p, data=d, headers=h, cookies=c
        )
        result = self.__get_json(resp)

        if self.__auto_report:
            if random.random() <= 0.1:  # 10%的概率触发
                self.data_report(ReportType.Source, None)

        if key:
            result = result.get(key)
        return(result)

    def _post_encrypt(self, url: str, data: dict, params: dict = None, headers: dict = None,
                      cookies: dict = None, key: str = 'result') -> dict:
        '''
        POST方法发送加密请求,data参数将被加密

        参数:
            url: URL
            [data]: 请求体
            [params]: 请求参数,会添加到self._params前面
            [headers]: 请求头,会替换self._headers
        返回:
            Response: 请求结果
        '''
        result = self.__post_encrypt_without_report(
            url=url,
            data=data,
            params=params,
            headers=headers,
            cookies=cookies,
            key=key)

        if self.__auto_report:
            if random.random() <= 0.1:  # 10%的概率触发
                self.data_report(ReportType.Source, None)

        if key:
            result = result.get(key)
        return(result)

    def __get_without_report(self, url: str, params: dict = None,  headers: dict = None,
                             cookies: dict = None, key: str = 'result') -> dict:
        '''
        GET方法发送请求[不自动调用report]

        参数:
            url: URL
            [params]: 请求参数,会添加到self._params前面
            [headers]: 请求头,会替换self._headers
            [cookies]: 请求头,会替换self._cookies
            [key]: 要返回的数据键名,默认为'result',留空表示返回原始json
        返回:
            dict: json字典
        '''
        self.__flash_token(url)
        p = {**(params or {}), **self.__params}
        h = headers or self.__headers
        c = cookies or self.__cookies
        resp = self.__session.get(
            url=url, params=p, headers=h, cookies=c
        )
        result = self.__get_json(resp)

        if key:
            result = result.get(key)
        return(result)

    def __post_encrypt_without_report(self, url: str, data: dict, params: dict = None, headers: dict = None,
                                      cookies: dict = None, key: str = 'result') -> dict:
        '''
        POST方法发送加密请求,data参数将被加密[不自动调用report]

        参数:
            url: URL
            [data]: 请求体
            [params]: 请求参数,会添加到self._params前面
            [headers]: 请求头,会替换self._headers
        返回:
            Response: 请求结果
        '''
        t = self.__flash_token(url)
        p = {**(params or {}), 'time_': t, **self.__params}
        h = headers or self.__headers
        c = cookies or self.__cookies
        d = encrypt_data(data, t)
        resp = self.__session.post(
            url=url, params=p, data=d, headers=h, cookies=c
        )
        result = self.__get_json(resp)
        if key:
            result = result.get(key)
        return(result)

    def _send_comment(self, linkid: int, message: str, ctype: int = 0, index: int = 0) -> bool:
        '''
        发送评论,通用

        参数:
            linkid: 文章id
            message: 文字评论内容
            [ctype]: 评论类型,参加static.CommentType
            [index]: 文章索引,模拟客户端行为
        返回:
            操作是否成功
        '''
        url = URLS.CREATE_COMMENT

        d = {'link_id': linkid, 'text': message,
             'root_id': -1, 'reply_id': -1, 'imgs': None}
        if ctype == CommentType.Roll:
            p = {}
        elif ctype in (CommentType.News, CommentType.Community):
            if ctype == CommentType.Community:
                hsrc = b64encode('bbs_app_feeds')
            elif ctype == CommentType.News:
                hsrc = b64encode('news_feeds_-1')
            p = {'h_src': hsrc, 'index': index}
        else:
            raise ValueError('ctype错误')
        try:
            result = self._post(url=url, data=d, params=p)
            self.logger.debug('发送评论成功')
            return(True)
        except ClientException as e:
            self.logger.error(f'发送评论出错 [{e}]')
            return(False)

    # def _get_comments(self, linkid: int, amount: int = 30,
    #                   author_only: bool = False) -> list:
    #     '''
    #     获取评论,通用

    #     参数:
    #         linkid: 文章id
    #         [amount]: 要拉取的数量
    #     返回:
    #         list: [(commintid,text,userid)…],评论列表
    #     '''
    #     pass

    # def _like_content(self):
    #     '''
    #     点赞,通用
    #     '''
    #     pass



'''
# @Author       : Chr_
# @Date         : 2020-07-16 15:54:49
# @LastEditors  : Chr_
# @LastEditTime : 2020-08-21 19:00:46
# @Description  : 静态资源
'''

# PYXIAOHEIHE版本
PYXIAOHEIHE_VERSION = '1.0.5'

# 小黑盒客户端版本号
HEYBOX_VERSION = '1.3.122'

# 遇到空结果继续请求的次数
EMPTY_RETRYS = 0

# 遇到错误继续请求的次数
ERROR_RETRYS = 1


class URLS():
    'URL常量'
    # 旧版遗留API接口,未必都能正常工作
    HEYBOX_VERSION_CHECK = 'https://api.xiaoheihe.cn/account/version_control_info/'  # 检查更新
    SCRIPT_UPDATE_CHECK = 'https://api.github.com/repos/chr233/xhh_auto/releases/latest'  # 脚本更新检查

    GET_TASK_STATS = 'https://api.xiaoheihe.cn/task/stats/'  # 任务状态
    GET_TASK_LIST = 'https://api.xiaoheihe.cn/task/list/'  # 任务列表
    GET_FOLLOW_ALERT = 'https://api.xiaoheihe.cn/bbs/app/api/follow/alert'  # 关注更新提醒

    # GET_NEWS_DETAIL = 'https://api.xiaoheihe.cn/maxnews/app/detail/'  # 文章页
    GET_GAME_COMMENTS = 'https://api.xiaoheihe.cn/bbs/app/link/game/comments/'  # 游戏评价
    GET_GAME_DETAIL = 'https://api.xiaoheihe.cn/game/get_game_detail/'  # 游戏详情
    GET_GAME_REVIEWS = 'https://api.xiaoheihe.cn/bbs/app/link/game/reviews'  # 游戏文章
    GET_RECOMMEND_FOLLOWING = 'https://api.xiaoheihe.cn/bbs/app/profile/recommend/following'  # 拉取推荐关注列表
    GET_ADS_INFO = 'https://api.xiaoheihe.cn/account/get_ads_info/'  # 拉取广告
    GET_FOLLOW_ALERT = 'https://api.xiaoheihe.cn/bbs/app/api/follow/alert'  # 关注列表更新提醒
    GET_AUTH_INFO = 'https://api.xiaoheihe.cn/account/get_auth_info/'  # 获取账户验证信息

    GET_COMMUNITY_SURVEY = 'https://api.xiaoheihe.cn/bbs/app/api/activity/community_survey'  # 拉取社区答题题目
    GET_ACHIEVE_LIST = 'https://api.xiaoheihe.cn/bbs/app/profile/achieve/list'  # 检查有没有解锁新成就
    GET_NOTIFY_ALERT = 'https://api.xiaoheihe.cn/bbs/app/api/notify/alert'  # 私信/通知提醒
    GET_USER_POST = 'https://api.xiaoheihe.cn/bbs/app/profile/user/link/list'  # 拉取用户发帖
    GET_USER_COMMENT = 'https://api.xiaoheihe.cn/bbs/app/profile/news/comment/list'  # 拉取用户评论
    GET_STORE_GAME_LIST = 'https://api.xiaoheihe.cn/mall/list/'  # 获取商店的游戏列表

    BBS_QA_STATE = 'https://api.xiaoheihe.cn/task/push_bbs_qa_state/'  # 社区答题提交
    UPDATE_PROFILE = 'https://api.xiaoheihe.cn/account/update_profile/'  # 修改个人资料
    SEND_MESSAGE = 'https://api.xiaoheihe.cn/chat/send_message/'  # 发送私信

    # 调试用
    HTTPBIN_GET = 'https://httpbin.org/get'
    HTTPBIN_POST = 'https://httpbin.org/post'

    # 验证过的API
    SIGN = 'https://api.xiaoheihe.cn/task/sign/'  # 签到
    GET_NEWS = 'https://api.xiaoheihe.cn/bbs/app/feeds/news'   # 获取新闻列表
    GET_TAGS = 'https://api.xiaoheihe.cn/maxnews/app/tag/list'  # 获取文章标签
    GET_COMMENTS = 'https://api.xiaoheihe.cn/bbs/app/link/tree'  # 获取文章附加信息
    LIKE_COMMENT = 'https://api.xiaoheihe.cn/bbs/app/comment/support'  # 评论点赞
    SUPPORT_COMMENT = 'https://api.xiaoheihe.cn/bbs/app/link/game/comment/up'  # 评测点赞
    GET_RECOMMEND_POST = 'https://api.xiaoheihe.cn/bbs/app/feeds'  # 获取社区推荐帖子

    LIKE_LINK = 'https://api.xiaoheihe.cn/bbs/app/profile/award/link'  # 一般点赞
    SHARE_CLICK = 'https://api.xiaoheihe.cn/bbs/app/link/share/click'  # 分享
    SHARE_CHECK = 'https://api.xiaoheihe.cn/task/shared/'  # 检查分享
    CREATE_COMMENT = 'https://api.xiaoheihe.cn/bbs/app/comment/create'  # 评论

    GET_FAN_LIST = 'https://api.xiaoheihe.cn/bbs/app/profile/follower/list'  # 拉取粉丝列表
    GET_FOLLOW_LIST = 'https://api.xiaoheihe.cn/bbs/app/profile/following/list'  # 拉取关注列表

    FOLLOW_USER = 'http://api.xiaoheihe.cn/bbs/app/profile/follow/user'  # 加关注
    UNFOLLOW_USER = 'https://api.xiaoheihe.cn/bbs/app/profile/follow/user/cancel'  # 取消关注

    GET_USER_PROFILE = 'https://api.xiaoheihe.cn/bbs/app/profile/user/profile'  # 获取个人资料

    GET_NEWS_CONTENT = 'https://api.xiaoheihe.cn/bbs/app/link/web/view'  # 视频页框架

    GET_ROLL_ROOM = 'https://api.xiaoheihe.cn/store/get_all_active_roll_room/'  # 拉取ROLL房列表
    JOIN_ROLL_ROOM = 'https://api.xiaoheihe.cn/store/join_roll_room/'  # 发送私信

    GET_SUBSCRIB_EVENTS = 'https://api.xiaoheihe.cn/bbs/app/profile/subscribed/events'  # 关注列表
    GET_USER_EVENTS = 'https://api.xiaoheihe.cn/bbs/app/profile/events'  # 拉取用户动态
    GET_UNREAD_MESSAGE = 'https://api.xiaoheihe.cn/bbs/app/user/message'  # 获取未读消息数量

    GET_MY_CARD = 'https://api.xiaoheihe.cn/mall/coupons/'  # 获取我的卡券

    LOGIN = 'https://api.xiaoheihe.cn/account/login/'  # 登录账号

    DATA_REPORT = 'https://api.xiaoheihe.cn/account/data_report/'  # 数据汇报


class TAGS():
    '标签转换工具类'
    name2key = {'PC游戏': 'evaluating', '宝可梦': 'Pokemon', '主机游戏': 'console', '桌游综合': 'boardgame',
                '荒野大镖客2': 'topic_53575', '云顶之弈': 'Teamfight', '逃离塔科夫': 'topic_74681',
                '死亡搁浅': 'topic_death', '荒野乱斗': 'huangyeluandou', 'Wallpaper': 'topic_11635',
                '超猎都市': 'topic_417725', '盗贼之海': 'topic_415163', '明日方舟': 'topic_23799',
                '战双帕弥什': 'ZSPMS', '和平精英': 'HPJY', '刀塔霸业': 'DotaUnderlords', '最后生还者系列': 'tloutwo',
                '对马之魂': 'topic_77355', '原神': 'Genshin', '手机游戏': 'Mobile', '盒友杂谈': 'HotSpots',
                '使命召唤': 'cod', '刺客信条系列': 'topic_50971', '绝地求生': 'PUBG', '数码硬件': 'event ',
                '命运2': 'Destiny', 'CS:GO': 'csgo', '彩虹六号': 'R6', '守望先锋': 'Blizzard', '英雄联盟': 'lol',
                '侠盗猎车手5': 'topic_2614', '怪物猎人': 'Mhw', '刀塔自走棋': 'DOTA_Piece', '炉石传说': 'hs',
                '公主连结': 'pcr', '王者荣耀': 'wzry', '骑马与砍杀2': 'dive', 'Valorant': 'topic_235709',
                '刀塔2': 'dota2', '魔兽世界': 'WOW', '动物森友会': 'animal'}
    key2name = {'evaluating': 'PC游戏', 'Pokemon': '宝可梦', 'console': '主机游戏', 'boardgame': '桌游综合',
                'topic_53575': '荒野大镖客2', 'Teamfight': '云顶之弈', 'topic_74681': '逃离塔科夫',
                'topic_death': '死亡搁浅', 'huangyeluandou': '荒野乱斗', 'topic_11635': 'Wallpaper',
                'topic_417725': '超猎都市', 'topic_415163': '盗贼之海', 'topic_23799': '明日方舟',
                'ZSPMS': '战双帕弥什', 'HPJY': '和平精英', 'DotaUnderlords': '刀塔霸业', 'tloutwo': '最后生还者系列',
                'topic_77355': '对马之魂', 'Genshin': '原神', 'Mobile': '手机游戏', 'HotSpots': '盒友杂谈',
                'cod': '使命召唤', 'topic_50971': '刺客信条系列', 'PUBG': '绝地求生', 'event ': '数码硬件',
                'Destiny': '命运2', 'csgo': 'CS: GO', 'R6': '彩虹六号', 'Blizzard': '守望先锋', 'lol': '英雄联盟',
                'topic_2614': '侠盗猎车手5', 'Mhw': '怪物猎人', 'DOTA_Piece': '刀塔自走棋', 'hs': '炉石传说',
                'pcr': '公主连结', 'wzry': '王者荣耀', 'dive': '骑马与砍杀2', 'topic_235709': 'Valorant',
                'dota2': '刀塔2', 'WOW': '魔兽世界', 'animal': '动物森友会'}


class RecTagType():
    '推荐关注列表用户类型分类'
    UnknownType = 0  # 未知
    SteamFriend = 1  # Steam好友
    HasFriend = 2  # 多位共同好友
    Author = 3  # 作者


class RelationType():
    '好友关系分类'
    NoRelation = 0  # 没有关系
    IFollowedHim = 1  # 我关注他
    HeFollowedMe = 2  # 他关注我
    BOthFollowed = 3  # 互关
    Unknown = 4
    num2name = {
        0: '没有关系', 1: '我关注他', 2: '他关注我', 3: '互关', 4: '错误'
    }


class CardType():
    '卡券分类'
    Unknown = 0
    Cut = 1
    Game = 2
    num2name = {
        0: '错误', 1: '优惠券', 2: '礼品卡'
    }


class NewsContentType():
    '首页新闻类型'
    Unknown = 0  # 未知
    VideoNews = 1  # 视频内容
    TextNews = 2  # 普通新闻
    TextNewsEx = 4  # 普通新闻,但是有大占位
    Recommand = 5  # 您是否对XXX感兴趣?'
    MultipleNews = 7  # 新闻专题
    Banner = 23  # 首页Banner


class EventType():
    '动态文章类型'
    Unknown = 0  # 未知
    PostLink = 1  # 发帖
    FollowGame = 2  # 关注游戏
    PurchaseGame = 3  # 购买游戏
    AchieveGame = 4  # 获得成就【疑似被弃用】
    CommentGame = 5  # 评测游戏
    CreateRollRoom = 6  # 赠送游戏
    name2num = {'Unknown': 0, 'post_link': 1,
                'follow_game': 2, 'game_purchase': 3,
                'game_achieve': 4, 'game_comment': 5,
                'roll_room': 6}


class RollSort():
    'Roll房排序'
    ROLL = 0  # 最近开奖
    TIME = 1  # 最新创建
    PRICE = 2  # 最高价格
    num2name = {0: 'roll', 1: 'time', 2: 'price'}


class GamePlatformType():
    '游戏平台类型'
    UnknownType = 0  # 未知
    PCGame = 1  # PC游戏
    ConsoleGame = 2  # 主机游戏


class GameReviewSummaryType():
    '游戏评价类型'
    UnknownType = 0  # 未知
    No = 1  # 无总体评价
    PPPP = 2  # 好评如潮
    PPP = 3  # 特别好评
    PP = 4  # 多半好评
    PN = 5  # 褒贬不一
    NP = 5  # 褒贬不一
    NN = 6  # 多半差评
    NNN = 7  # 特别差评
    NNNN = 8  # 差评如潮


class StateType():
    '答题结果'
    Complete = 1  # 答题完成
    AlreadyDone = 2  # 已经答过了,无法重复作答


class CommentType():
    '评论类型'
    News = 0  # 新闻评论
    Community = 1  # 社区评论
    Roll = 2  # Roll房评论


class ReportType():
    '汇报类型'
    Source = 1  # source模板
    Quit = 2  # events模板
    View = 3    # views模板


class BString():
    '逻辑型字符串,可以打印出×和√,不影响逻辑判断'
    __boolen = False
    __string = '×'

    def __init__(self, boolen):
        super().__init__()
        self.__boolen = bool(boolen)
        self.__string = '√' if boolen else '×'

    def __eq__(self, value):
        if isinstance(value, BString):
            return self.__boolen == value.__boolen
        elif isinstance(value, bool):
            return (self.__boolen == value)
        elif isinstance(value, str):
            return (self.__string == value)
        else:
            return(False)

    def __bool__(self):
        return(self.__boolen)

    def getbool(self):
        return(self.__boolen)

    def __hash__(self):
        return hash(self.__boolen)

    def __str__(self):
        return(self.__string)

    def __repr__(self):
        return(self.__string)


# User-Agent
Android_UA = 'Mozilla/5.0 AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2272.118 Safari/537.36 ApiMaxJia/1.0'
iOS_UA = 'xiaoheihe/1.3.119 (iPhone; iOS %s; Scale/3.00)'

# 加密相关
RSA_PUB_KEY = '''-----BEGIN PUBLIC KEY-----
MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQDZgjVwAiKTjZ55nG+mW6r3TSU4
ECvNYqDMIS/bhCj2QaH5GI/KZb2TBp+CBvUj9SLFnmJQ0kzHzHoGZCQ88VevCffF
7JePGF9cmKQqotlfTKbV4oxV5iLz7JSG6b/Vg7AXtrTolNtWsa8HiB0tI0YClYaQ
lOXm4UxLeSxQwSFETwIDAQAB
-----END PUBLIC KEY-----'''

DES_IV = 'abcdefgh'

ENC_STATIC = '//Z1q/Gb/R///+9xZ561TtoHjPrv2ew0Ln8vZnI5oObw+++oa3zw++1yd7wMqU/eNKahfmji5/xDu7EuCQfjaRk4TBKXrnhrlnkz@%$^&*(-_-)hahaha(-_-)_time='



'''
# @Author       : Chr_
# @Date         : 2020-07-30 16:29:29
# @LastEditors  : Chr_
# @LastEditTime : 2020-08-20 17:53:32
# @Description  : 游戏模块,负责[游戏库]TAB下的内容
'''

from .network import Network
from .static import RollSort, URLS, CommentType, ERROR_RETRYS, EMPTY_RETRYS, CardType
from .utils import ex_extend
from .error import ClientException, Ignore


class Game(Network):

    def __init__(self, account: dict, hbxcfg: dict, debug: bool):
        super().__init__(account, hbxcfg, debug)

    def debug(self):
        super().debug()
        # self.get_roll_room(1)

    def get_roll_room(self, amount: int = 30, sort: int = 0,
                      ignore_joined: bool = True, ignore_password: bool = True) -> list:
        '''
        拉取可参与的ROLL房列表

        参数:
            [amount]: 要拉取的数量
            [sort]: 排序方式,参见static.RollSort
            [ignore_joined]: 是否忽略参加过的房间
            [ignore_password]: 是否忽略有密码的房间
        成功返回:
            list: 房间列表[(room_id,link_id,有密码?,已参与?),……]
        '''
        def get(offset=0):
            params = {'filter_passwd': '1', 'sort_types': RollSort.num2name.get(sort, 'roll'),
                      'page_type': 'home', 'offset': offset,   'limit': '30'}
            result = self._get(url=url, params=params)
            tmp = []
            for r in result['rooms']:
                try:
                    room_id = r['room_id']
                    link_id = r['link_id']
                    # people = room['people']
                    # price = room['price']
                    joined = 'joined' in r
                    password = r['has_pass']
                    # 三个过滤器
                    if 'over' not in r:
                        if password == False or ignore_password == False:
                            if joined == False or ignore_joined == False:
                                tmp.append(
                                    (room_id, link_id, password, joined))
                    else:
                        break
                except KeyError as e:
                    self.logger.debug(f'提取ROLL房出错[{r}]')
            self.logger.debug(f'拉取[{len(tmp)}]个ROLL房')
            return(tmp)
        # ==========================================
        url = URLS.GET_ROLL_ROOM
        roomlist = []
        empty = 0
        error = 0
        for i in range(0, amount//30 + 3):
            try:
                self.logger.debug(f'拉取第[{i+1}]批ROLL房列表')
                tmp = get(i * 30)
                if tmp:
                    roomlist = ex_extend(roomlist, tmp)
                else:
                    self.logger.debug('[*] ROLL房列表为空,可能没有可参与的ROLL房,也可能遇到错误')
                    empty += 1
                    if empty > EMPTY_RETRYS:
                        self.logger.debug('[*] 空结果达到上限,停止操作')
                        break
                if len(roomlist) >= amount:
                    break
            except ClientException as e:
                self.logger.debug(f'[*] 拉取ROLL房出错[{e}]')
                error += 1
                if error > ERROR_RETRYS:
                    self.logger.error('[*] 错误次数达到上限,停止操作')

        roomlist = roomlist[:amount]
        if len(roomlist) > 0:
            self.logger.debug(f'操作完成,拉取了[{len(roomlist)}]个ROLL房')
        else:
            self.logger.debug('拉取完毕,ROLL房列表为空,可能没有可参与的ROLL房')
        return(roomlist)

    def like_roll_room(self, linkid: int,  like: bool = True) -> bool:
        '''
        给好友动态点赞

        参数:
            linkid: 文章id
            [like]: 为True表式点赞,False表式取消赞
        返回:
            bool: 操作是否成功
        '''
        data = {'link_id': linkid, 'award_type': 1 if like else 0}
        url = URLS.LIKE_LINK
        try:
            result = self._post(url=url, data=data)
            self.logger.debug('房间点赞/取消点赞成功')
            return(True)
        except Ignore:
            self.logger.debug('房间已经点赞/取消点赞了')
            return(True)
        except ClientException as e:
            self.logger.error(f'房间点赞/取消点赞出错 [{e}]')
            return(False)

    def send_roll_comment(self, linkid: int, message: str) -> bool:
        '''
        发送Roll房评论

        参数:
            linkid: 文章id
            message: 文字评论内容
        返回:
            操作是否成功
        '''
        result = self._send_comment(
            linkid, message, CommentType.Roll, 0)
        return(result)

    def get_roll_comments(self, linkid: int, amount: int = 30,
                          author_only: bool = False) -> list:
        '''
        拉取Roll房的评论列表,不包含楼中楼,失败返回False

        参数:
            linkid: 文章id
            [amount]: 要拉取的数量
            [author_only]: 是否开启只看楼主
        返回:
            list: [(commintid,text,userid)…],评论列表
        '''
        def get(page: int) -> list:
            params = {'link_id': linkid, 'page': page, 'limit': 30,
                      'is_first': 1 if page == 1 else 0,
                      'owner_only': 1 if author_only else 0}
            result = self._get(url=url, params=params)
            tmp = []
            for cmt in result['comments']:
                try:
                    c = cmt['comment'][0]
                    # child_num = comment['child_num'] #回复数
                    commentid = c['commentid']  # 评论ID
                    text = c['text']  # 评论内容
                    u = c['user']
                    username = u['username']
                    userid = u['userid']
                    # level = comment['level_info']['level'] #等级
                    # self.logger.debug(
                    #     f'[{username}][{userid}]：[{commentid}][{text}]')
                    tmp.append((commentid, userid, text))
                except KeyError:
                    self.logger.error(f'[*] 拉取房间评论出错 [{cmt}]')
            self.logger.debug(f'拉取[{len(tmp)}]条评论')
            return(tmp)
        # ==========================================
        url = URLS.GET_COMMENTS
        commentslist = []
        empty = 0
        error = 0
        for i in range(0, amount//30 + 2):
            try:
                self.logger.debug(f'拉取第[{i+1}]页评论')
                tmp = get(i)
                if tmp:
                    commentslist.extend(tmp)
                else:
                    self.logger.debug('评论列表为空,可能遇到错误')
                    empty += 1
                    if empty > EMPTY_RETRYS:
                        self.logger.debug('空结果达到上限,停止操作')
                        break
                if len(commentslist) >= amount:
                    break
            except ClientException as e:
                self.logger.debug(f'拉取评论列表出错[{e}]')
                error += 1
                if error > ERROR_RETRYS:
                    self.logger.debug('错误次数达到上限,停止操作')
                    break

        commentslist = commentslist[:amount]
        if len(commentslist) > 0:
            self.logger.debug(f'操作完成,拉取了[{len(commentslist)}]条评论')
        else:
            self.logger.debug('[*] 拉取完毕,评论列表为空,可能遇到错误')
        return(commentslist)

    def join_roll_room(self, room_id: int) -> bool:
        '''
        参加Roll房

        参数:
            room_id: roll房号
        返回:
            bool: 操作是否成功
        '''
        url = URLS.JOIN_ROLL_ROOM
        data = {"room_id": room_id}
        try:
            self._post_encrypt(url=url,  data=data)
            self.logger.debug('加入Roll房成功')
            return(True)
        except Ignore:
            self.logger.debug('重复加入Roll房')
            return(True)
        except ClientException as e:
            self.logger.error(f'[*] 加入Roll房出错 [{e}]')
            return(False)

    def get_my_card(self, amount: int = 30) -> list:
        '''
        拉取我的卡券

        参数:
            [amount]: 要拉取的数量
        成功返回:
            list: 卡券列表
        '''
        def get(offset=0):
            params = {'type': 0, 'cat': 'all',
                      'offset': offset,   'limit': '30'}
            result = self._get(url=url, params=params)
            tmp = []
            for l in result['list']:
                for i in l['list']:
                    try:
                        cardtype = i['type']
                        cardid = i['coupon_id']
                        name = i['name']
                        if cardtype == 'moneyoff':  # 优惠券
                            cardtype = CardType.Cut
                        elif cardtype == 'exchange':  # 礼品卡
                            cardtype = CardType.Game
                        else:
                            cardtype = CardType.Unknown
                        tmp.append((cardid, cardtype, name))
                    except KeyError as e:
                        self.logger.debug(f'提取卡券列表出错[{r}]')
            self.logger.debug(f'拉取[{len(tmp)}]个卡券')
            return(tmp)
        # ==========================================
        url = URLS.GET_MY_CARD
        cardlist = []
        empty = 0
        error = 0
        for i in range(0, amount//30 + 3):
            try:
                self.logger.debug(f'拉取第[{i+1}]批卡券列表')
                tmp = get(i * 30)
                if tmp:
                    cardlist = ex_extend(cardlist, tmp)
                else:
                    self.logger.debug('[*] 卡券列表为空,可能没有可用卡券,也可能遇到错误')
                    empty += 1
                    if empty > EMPTY_RETRYS:
                        self.logger.debug('[*] 空结果达到上限,停止操作')
                        break
                if len(cardlist) >= amount:
                    break
            except ClientException as e:
                self.logger.debug(f'[*] 拉取卡券列表出错[{e}]')
                error += 1
                if error > ERROR_RETRYS:
                    self.logger.error('[*] 错误次数达到上限,停止操作')

        cardlist = cardlist[:amount]
        if len(cardlist) > 0:
            self.logger.debug(f'操作完成,拉取了[{len(cardlist)}]个卡券')
        else:
            self.logger.debug('拉取完毕,卡券列表为空,可能没有可用卡券')
        return(cardlist)



'''
# @Author       : Chr_
# @Date         : 2020-07-30 16:28:55
# @LastEditors  : Chr_
# @LastEditTime : 2020-08-08 19:47:16
# @Description  : 首页模块,负责[首页]TAB下的内容
'''

from .network import Network
from .static import URLS, BString, EMPTY_RETRYS, ERROR_RETRYS, EventType, NewsContentType
from .error import ClientException, Ignore
from .utils import ex_extend, b64encode


class Index(Network):

    def __init__(self, account: dict, hbxcfg: dict, debug: bool):
        super().__init__(account, hbxcfg, debug)

    def debug(self):
        super().debug()

    def get_news(self, amount: int = 30, tag: str = '-1') -> list:
        '''
        获取首页文章列表

        参数:
            [amount]: 需要拉取的数量
            [tag]: 文章分区,可以参考static.TAGS类,-1代表首页
        成功返回:
            list: 文章id列表,[(linkid,title,desc,userid),……]
        '''
        def get(offset: int) -> list:
            params = {'offset': offset, 'limit': 30,
                      'tag': tag, 'rec_mark': 'timeline'}
            result = self._get(url=url, params=params)
            tmp = []
            for l in result['links']:
                if l['content_type'] in(NewsContentType.VideoNews,
                                        NewsContentType.TextNews,
                                        NewsContentType.TextNewsEx):
                    tmp.append((l['linkid'], l['title'],
                                l['description'], l.get('userid', 0)))
            self.logger.debug(f'拉取了{len(tmp)}条新闻')
            return(tmp)
        # ==========================================
        url = URLS.GET_NEWS
        newslist = []
        empty = 0
        error = 0
        for i in range(0, amount//30 + 2):
            try:
                self.logger.debug(f'拉取第[{i+1}]批新闻')
                tmp = get(i * 30)
                if tmp:
                    newslist.extend(tmp)
                else:
                    self.logger.debug('[*] 新闻列表为,可能遇到错误')
                    empty += 1
                    if empty > EMPTY_RETRYS:
                        self.logger.debug('[*] 空结果达到上限,停止操作')
                        break
                if len(newslist) >= amount:
                    break
            except ClientException as e:
                self.logger.debug(f'[*] 拉取首页文章列表出错 [{e}]')
                error += 1
                if error > ERROR_RETRYS:
                    self.logger.debug('[*] 错误次数达到上限,停止操作')
                    break
        newslist = newslist[:amount]
        if len(newslist) > 0:
            self.logger.debug(f'操作完成,拉取了[{len(newslist)}]条新闻')
        else:
            self.logger.warning('[*] 拉取完毕,新闻列表为空,请检查参数')
        return(newslist)

    def get_news_id(self, amount: int = 30, tag: str = '-1') -> list:
        '''
        获取首页文章id列表

        参数:
            [amount]: 需要拉取的数量
            [tag]: 文章分区,可以参考static.TAGS类,-1代表首页
        成功返回:
            list: 文章id列表,[linkid,……]
        '''
        newslist = self.get_news(amount, tag)
        idlist = [x[0] for x in newslist]
        return(idlist)

    def get_news_content(self, linkid: int, index: int = 1) -> (str, dict):
        '''
        拉取新闻正文

        参数:
            linkid: 文章id
            [index]: 可以理解为文章排在首页的位置,banner为0,从上往下依次递增
        成功返回:
            str: 文章标题
            dict: 文章内容字典,类似富文本
        '''
        url = URLS.GET_NEWS_CONTENT

        params = {'return_json': 1, 'index': index,
                  'h_src': b64encode('news_feeds_-1'), 'link_id': linkid}
        if index == 0:
            params.pop('index')
            params.pop('h_src')
        try:
            result = self._get(url=url, params=params)
            l = result['link']
            title = l['title']
            content = l['content']
            return((title, content))
        except ClientException as e:
            self.logger.error(f'拉取文章出错[{e}]')
            return(False)

    def get_comments(self, linkid: int, amount: int = 30,
                     index: int = 1, author_only: bool = False) -> list:
        '''
        拉取文章的评论列表,不包含楼中楼,失败返回False

        参数:
            linkid: 文章id
            [amount]: 要拉取的数量
            [index]: 可以理解为文章排在首页的位置,banner为0,从上往下依次递增
            [author_only]: 是否开启只看楼主
        返回:
            list: [(commintid,text,userid)…],评论列表
        '''
        def get(page: int) -> list:
            params = {'h_src': b64encode('news_feeds_-1'),
                      'link_id': linkid, 'page': page, 'limit': 30,
                      'is_first': 1 if page == 1 else 0,
                      'owner_only': 1 if author_only else 0,
                      'index': index}
            if page != 1:
                params['sort_filter'] = 'hot'
            result = self._get(url=url, params=params)
            tmp = []
            for cmt in result['comments']:
                try:
                    c = cmt['comment'][0]
                    # child_num = comment['child_num'] #回复数
                    commentid = c['commentid']  # 评论ID
                    text = c['text']  # 评论内容
                    u = c['user']
                    username = u['username']
                    userid = u['userid']
                    # level = comment['level_info']['level'] #等级
                    # self.logger.debug(
                    #     f'[{username}][{userid}]：[{commentid}][{text}]')
                    tmp.append((commentid, userid, text))
                except KeyError:
                    self.logger.error(f'[*] 拉取文章评论出错 [{cmt}]')
            self.logger.debug(f'拉取[{len(tmp)}]条评论')
            return(tmp)
        # ==========================================
        url = URLS.GET_COMMENTS
        commentslist = []
        empty = 0
        error = 0
        for i in range(0, amount//30 + 2):
            try:
                self.logger.debug(f'拉取第[{i+1}]页评论')
                tmp = get(i)
                if tmp:
                    commentslist.extend(tmp)
                else:
                    self.logger.debug('评论列表为空,可能遇到错误')
                    empty += 1
                    if empty > EMPTY_RETRYS:
                        self.logger.debug('空结果达到上限,停止操作')
                        break
                if len(commentslist) >= amount:
                    break
            except ClientException as e:
                self.logger.debug(f'拉取评论列表出错[{e}]')
                error += 1
                if error > ERROR_RETRYS:
                    self.logger.debug('错误次数达到上限,停止操作')
                    break

        commentslist = commentslist[:amount]
        if len(commentslist) > 0:
            self.logger.debug(f'操作完成,拉取了[{len(commentslist)}]条评论')
        else:
            self.logger.debug('[*] 拉取完毕,评论列表为空,可能遇到错误')
        return(commentslist)

    def get_comments_id(self, linkid: int, amount: int = 30,
                        index: int = 1, author_only: bool = False) -> list:
        '''
        拉取文章的评论id列表,失败返回False

        参数:
            linkid: 文章id
            [amount]: 要拉取的数量
            [index]: 可以理解为文章排在首页的位置,banner为0,从上往下依次递增
            [author_only]: 是否开启只看楼主
        返回:
            list: [commintid…],评论id列表
        '''
        commentslist = self.get_comments(linkid, amount, index, author_only)
        idlist = [x[0] for x in commentslist]
        return(idlist)

    def get_subscrib_events(self, amount: int = 30, ignore_liked: bool = True) -> list:
        '''
        拉取动态列表,失败返回False

        参数:
            value: 要拉取的数量
            [ignore_liked]: 忽略已点赞的动态?
        成功返回:
            list: [(linkid,ftype,已点赞?),……] ftype释义参见static.EventType
        '''
        def get(offset: int, lastval: str) -> (list, str):
            params = {'offset': offset, 'limit': 30, 'lastval': lastval,
                      'filters': 'post_link|follow_game|game_purchase|game_comment|roll_room'}
            if not lastval:
                params.pop('lastval')
            result = self._get(url=url, params=params)
            tmp = []
            val = result.get('lastval')
            if result['moments']:
                for m in result['moments']:
                    try:
                        l = m['link']
                        linkid = int(l['linkid'])
                        liked = bool(l['is_award_link'] == 1)
                        userid = int(m['user']['userid'])
                        ftype = EventType.name2num.get(
                            m['content_type'], 0)
                        # 过滤自己的评测
                        if ftype == EventType.CommentGame and userid == self.heybox_id:
                            continue
                        # 忽略已点赞的动态
                        if liked == False or ignore_liked == False:
                            tmp.append((linkid, ftype, liked))
                    except KeyError:
                        self.logger.debug(f'[*] 提取动态列表出错 [{m}]')
                self.logger.debug(f'拉取了[{len(tmp)}]条动态')
            else:
                self.logger.debug(f'服务器未返回动态数据,跳过本轮')
            return((tmp, val))
        # ==========================================
        url = URLS.GET_SUBSCRIB_EVENTS
        eventslist = []
        error = 0
        empty = 0
        lastval = None
        lastcount = 0  # 记录动态条数
        for i in range(0, amount//30+4):
            try:
                self.logger.debug(f'拉取第[{i+1}]批动态')
                tmp, lastval = get(i * 30, lastval)
                if not lastval:  # 服务器未返回数据
                    continue
                if tmp:
                    eventslist = ex_extend(eventslist, tmp)
                else:
                    self.logger.debug('[*] 动态列表为空,可能遇到错误')
                    empty += 1
                    if empty > EMPTY_RETRYS:
                        self.logger.debug('[*] 空结果达到上限,停止操作')
                        break
                count = len(eventslist)
                if count >= amount:
                    break
                if count == lastcount:
                    self.logger.debug('[*] 没有更多动态,停止操作')
                    break
                else:
                    lastcount = len(eventslist)
            except ClientException as e:
                self.logger.debug(f'[*] 拉取动态列表出错 [{e}]')
                error += 1
                if error > ERROR_RETRYS:
                    self.logger.debug('[*] 错误次数达到上限,停止操作')
                    break

        eventslist = eventslist[:amount]
        if len(eventslist) > 0:
            self.logger.debug(f'操作完成,拉取了[{len(eventslist)}]条动态')
        else:
            self.logger.debug('拉取完毕,动态列表为空,可能遇到错误')
        return(eventslist)

    def get_user_enevts(self, userid: int, amount: int = 30, ignore_liked: bool = True):
        '''拉取指定用户动态列表,失败返回False
        参数:
            userid: 用户id
            amount: 要拉取的数量
            ignoreliked: 忽略已点赞?
        返回:
            list: [(linkid,ftype,已点赞?),……] ftype释义参见static.EventType
            '''
        def get(offset: int, lastval: str) -> (list, str):
            params = {'userid': userid, 'offset': offset,
                      'limit': 30, 'lastval': lastval}
            if not lastval:
                params.pop('lastval')
            result = self._get(url=url, params=params)
            tmp = []
            val = result.get('lastval')
            for m in result['moments']:
                try:
                    link = m['link']
                    linkid = int(link['linkid'])
                    liked = bool(link['is_award_link'] == 1)
                    ftype = EventType.name2num.get(
                        m['content_type'], 0)
                    # 过滤自己的评测
                    if ftype == EventType.CommentGame and userid == self.heybox_id:
                        continue
                    # 忽略已点赞的动态
                    if liked == False or ignore_liked == False:
                        tmp.append((linkid, ftype, liked))
                except KeyError:
                    self.logger.debug(f'提取动态列表出错[{m}]')
            self.logger.debug(f'拉取[{len(tmp)}]条动态')
            return((tmp, val))
        # ==========================================
        url = URLS.GET_USER_EVENTS
        eventslist = []
        empty = 0
        error = 0
        lastval = None
        for i in range(0, amount//30+4):
            try:
                self.logger.debug(f'拉取第[{i+1}]批动态列表')
                tmp, lastval = get(i * 30, lastval)
                if tmp:
                    eventslist = ex_extend(eventslist, tmp)
                else:
                    self.logger.debug('[*] 动态列表为空,可能遇到错误')
                    empty += 1
                    if empty > EMPTY_RETRYS:
                        self.logger.debug('[*] 空结果达到上限,停止操作')
                        break
                if len(eventslist) >= amount:
                    break
            except ClientException as e:
                self.logger.debug(f'[*] 拉取动态列表出错 [{e}]')
                error += 1
                if error > ERROR_RETRYS:
                    self.logger.error('[*] 错误次数达到上限,停止操作')
                    break

        eventslist = eventslist[:amount]
        if len(eventslist) > 0:
            self.logger.debug(f'操作完成,拉取了[{len(eventslist)}]条评论')
        else:
            self.logger.debug('拉取完毕,评论列表为空,可能遇到错误')
        return(eventslist)

    def get_tags(self) -> list:
        '''
        获取标签列表,可以用于获取指定分区的文章,出错返回False

        返回:
            list: tag列表,[(name,key),…]
        '''
        url = URLS.GET_TAGS
        try:
            result = self._get(url)
            t = result['tags'][1:]
            tags = [(x['tag'], x['key']) for x in t]
            return(tags)
        except ClientException as e:
            self.logger.error(f'[*] 获取标签列表出错 [{e}]')

    def like_comment(self, commentid: int, like: bool = True) -> bool:
        '''给评论点赞

        参数:
            commentid: 评论id
            [like]: True点赞/False取消赞
        返回:
            bool: 操作是否成功
        '''
        url = URLS.LIKE_COMMENT
        data = {'comment_id': commentid, 'support_type': 1 if like else 2}
        try:
            self._post(url=url, data=data)
            self.logger.debug('评论点赞/取消点赞成功')
            return(True)
        except Ignore:
            self.logger.debug('已经点过赞了')
            return(True)
        except ClientException as e:
            self.logger.error(f'[*] 评论点赞/取消点赞出错,也有可能是重复取消点赞 [{e}]')
            return(False)

    def like_news(self, linkid: int, index: int = 1, like: bool = True) -> bool:
        '''
        给新闻点赞

        参数:
            linkid: 文章链接id
            [index]: 可以理解为文章排在首页的位置,banner为0,从上往下依次递增
            [like]: True点赞/False取消赞
        返回:
            bool: 操作是否成功
        '''
        url = URLS.LIKE_LINK
        data = {'link_id': linkid, 'award_type': 1 if like else 0}
        if index:
            params = {'h_src': b64encode('news_feeds_-1'), 'index': index}
        else:
            params = {}
        try:
            self._post(url=url, params=params, data=data)
            self.logger.debug('文章点赞/取消点赞成功')
            return(True)
        except Ignore:
            self.logger.debug('文章已经点赞/取消点赞了')
            return(True)
        except ClientException as e:
            self.logger.error(f'[*] 文章点赞/取消点赞出错 [{e}]')
            return(False)

    def like_event(self, linkid: int, ftype: int = 0, like: bool = True) -> bool:
        '''
        给好友动态点赞

        参数:
            linkid: 动态id
            [ftype]: 动态类型,释义参见static.FollowPostType
            [like]: True点赞/False取消赞
        返回:
            bool: 操作是否成功
        '''
        data = {'link_id': linkid}
        if ftype == EventType.CommentGame:
            url = URLS.SUPPORT_COMMENT
            data['support_type'] = 1 if like else 0
        else:
            url = URLS.LIKE_LINK
            data['award_type'] = 1 if like else 0
        try:
            result = self._post(url=url, data=data)
            self.logger.debug('动态点赞/取消点赞成功')
            return(True)
        except Ignore:
            self.logger.debug('动态已经点赞/取消点赞了')
            return(True)
        except ClientException as e:
            self.logger.error(f'动态点赞/取消点赞出错 [{e}]')
            return(False)

    def share_news(self, linkid: int, index: int = 1) -> bool:
        '''
        分享首页文章

        参数:
            newsid: 文章id
            [index]: 可以理解为文章排在首页的位置,banner为0,从上往下依次递增
        返回:
            bool: 是否成功
        '''
        def click():
            url = URLS.SHARE_CLICK
            params = {'h_src': b64encode('news_feeds_-1'),
                      'link_id': linkid, 'index': index}
            try:
                self._get(url=url, params=params)
                self.logger.debug('模拟点击分享按钮')
                return(True)
            except ClientException as e:
                self.logger.error(f'分享出错[{e}]')
                return(False)

        def check():
            url = URLS.SHARE_CHECK
            params = {'h_src': b64encode('news_feeds_-1'),
                      'shared_type': 'normal'}
            try:
                self._get(url=url, params=params)
                self.logger.debug('分享成功')
                return(True)
            except ClientException as e:
                self.logger.debug(f'分享出错(貌似还是可以完成任务) [{e}]')
                return(False)
        r1 = click()
        r2 = check()
        return(r1 and r2)

    def share_comment(self) -> bool:
        '''
        分享文章评论

        参数:
            不需要??
        返回:
            bool: 是否成功
        '''
        url = URLS.SHARE_CHECK
        params = {'shared_type': 'BBSComment'}
        try:
            self._get(url=url, params=params)
            self.logger.debug('分享成功')
            return(True)
        except ClientException as e:
            self.logger.debug(f'分享出错(貌似还是可以完成任务) [{e}]')
            return(True)  # 貌似也能完成任务,所以返回True

    def sign(self) -> bool:
        '''
        进行签到

        返回:
            bool: 是否成功
        '''
        url = URLS.SIGN
        try:
            r = self._get(url=url)
            # li = r['level_info']
            # exp = li['exp']
            # coin = li['coin']
            # # max_exp = li['max_exp']
            # level = li['level']
            sign_coin = r['sign_in_coin']
            sign_exp = r['sign_in_exp']
            sign_days = r['sign_in_streak']

            self.logger.debug(f'签到成功,连续[{sign_days}]天')
            self.logger.debug(f'获得[{sign_coin}]盒币,[{sign_exp}]经验')
            return(True)
        except Ignore:
            self.logger.debug('已经签过到了')
            return(True)
        except ClientException as e:
            self.logger.error(f'签到出错[{e}]')
            return(False)




'''
# @Author       : Chr_
# @Date         : 2020-07-30 13:32:04
# @LastEditors  : Chr_
# @LastEditTime : 2020-08-07 12:25:08
# @Description  : 小黑盒客户端模块
'''

from .account import Account
from .community import Community
from .game import Game
from .index import Index


class HeyBoxClient(Account, Community, Game, Index):

    '''
    Python实现的小黑盒客户端
    '''

    def __init__(self, account: dict, hbxcfg: dict, debug: bool = False):
        '''
        初始化

        参数:
            account: 账号字典,需要包含'pkey','imei'和'heybox_id'
            hbxcfg: 客户端设置字典,需要包含
            [debug]: 调试模式开关,打开后会输出调试日志
        '''
        super().__init__(account, hbxcfg, debug)

    def debug(self):
        '''
        仅供测试使用
        '''
        return super().debug()



'''
# @Author       : Chr_
# @Date         : 2020-07-30 16:29:34
# @LastEditors  : Chr_
# @LastEditTime : 2020-08-11 11:30:32
# @Description  : 账号模块,负责[我]TAB下的内容
'''


from .network import Network
from .static import HEYBOX_VERSION, URLS, BString, ERROR_RETRYS, EMPTY_RETRYS, RelationType
from .error import *
from .utils import ex_extend, user_relation_filter


class Account(Network):
    '账号模块,负责[我]TAB下的内容'

    def __init__(self, account: dict, hbxcfg: dict, debug: bool):
        super().__init__(account, hbxcfg, debug)

    def debug(self):
        super().debug()

    def get_heybox_latest_version(self) -> str:
        '''
        获取小黑盒最新版本号,失败返回False

        返回:
            str: 小黑盒版本号
        '''
        url = URLS.HEYBOX_VERSION_CHECK
        try:
            result = self._get(url=url)
            version = result['version']
            message = result['msg']
            self.logger.debug(f'小黑盒最新版本为[{version}] - [{message}]')
            return(version)
        except (ClientException, KeyError, NameError) as e:
            self.logger.error(f'获取小黑盒最新版本出错[{e}]')
            return(False)

    def get_user_relation(self, userid: int) -> int:
        '''
        获取用户和我的关系,返回值释义参考static.RelationType

        参数:
            userid: 用户id
        返回:
            int: 关注状态,释义参考static.RelationType
        '''
        if userid == self.heybox_id:
            self.logger.debug('[*] 无法查看自己和自己的关系')
            return(RelationType.BOthFollowed)
        url = URLS.GET_USER_PROFILE
        params = {'userid': userid}
        try:
            result = self._get(url=url, params=params)

            ad = result['account_detail']

            uid = ad['userid']  # 用户id
            uname = ad['username']  # 用户名
            relation = ad['bbs_info']['follow_status']  # 关注状态

            self.logger.debug(
                f'昵称:{uname} @{uid} [{RelationType.num2name[relation]}]')
            return(relation)
        except (ClientException, KeyError, NameError) as e:
            self.logger.error(f'[*] 获取用户关系出错 [{e}]')
            return(RelationType.Unknown)

    def get_user_profile(self, userid: int = 0) -> (int, int, int):
        '''
        获取个人资料,失败返回False

        参数:
            [userid]: 用户id,不填代入自己的id
        返回:
            follow_num:关注数
            fan_num:粉丝数
            awd_num:获赞数
        '''
        url = URLS.GET_USER_PROFILE
        params = {'userid': userid or self.heybox_id}
        try:
            result = self._get(url=url, params=params)

            ad = result['account_detail']
            bi = ad['bbs_info']

            follow = bi['follow_num']  # 关注
            fan = bi['fan_num']  # 粉丝
            like = bi['awd_num']  # 获赞
            level = ad['level_info']['level']  # 等级
            uid = ad['userid']  # 用户id
            uname = ad['username']  # 用户名

            self.logger.debug(f'昵称:{uname} >{uid}< [{level}级]')
            self.logger.debug(f'关注[{follow}] 粉丝[{fan}] 获赞[{like}]')
            return((follow, fan, like))
        except (ClientException, KeyError, NameError) as e:
            self.logger.error(f'[*] 获取用户详情出错 [{e}]')
            return((0, 0, 0))

    def get_unread_count(self) -> (int, int, int, int, int):
        '''
        获取未读通知计数,失败返回False

        返回:
            like: 新获赞
            comment: 新评论
            follow: 新粉丝
            notify: 新通知
            total: 总计
        '''
        url = URLS.GET_UNREAD_MESSAGE
        params = {'list_type': 2, 'offset': 0}
        try:
            result = self._get(url=url, params=params)
            muu = result['message_unread_num']
            like = muu['award']  # 新获赞
            comment = muu['comment']  # 新评论
            # developer = muu['developer']
            follow = muu['follow']  # 新粉丝
            # friend = muu['friend']  # 大概是私信
            notify = muu['notify']  # 优惠通知
            total = muu['total']  # 总计
            self.logger.debug(f'新获赞[{like}]新评论[{comment}]新粉丝[{follow}]')
            return((like, comment, follow, notify, total))
        except (ClientException, KeyError, NameError) as e:
            self.logger.error(f'[*] 获取任务详情出错 [{e}]')
            return(False)

    def __get_task_json(self) -> dict:
        '''
        获取任务详情json,出错返回False

        返回:
            dict: json字典
        '''
        url = URLS.GET_TASK_LIST
        result = self._get(url=url)
        return(result)

    def get_daily_task(self) -> (BString, BString, BString, BString):
        '''
        获取每日任务详情,失败返回(False,False,False,False)

        返回:
            sign: 签到?
            share_news: 分享新闻?
            share_comment: 分享评论?
            like:点赞?
        '''
        try:
            result = self.__get_task_json()
            tl = result['task_list'][0]['tasks']
            sign = BString(tl[0]['state'] == 'finish')
            share_news = BString(tl[1]['state'] == 'finish')
            share_comment = BString(tl[2]['state'] == 'finish')
            like = BString(tl[3]['state'] == 'finish')

            self.logger.debug(
                f"签到{sign}|分享{share_news}|{share_comment}|点赞{like}")
            return((sign, share_news, share_comment, like))
        except ClientException as e:
            self.logger.error(f'[*] 获取任务详情出错 [{e}]')
            return(False, False, False, False)

    def get_my_data(self) -> (str, int, (int, int, int), int):
        '''
        获取我的数据,出错返回False

        返回:
            username:昵称
            userid: userid
            coin:H币
            (level,exp,max_exp):(等级,经验,下级经验)
            sign_in_streak:连续签到天数
        '''
        url = URLS.GET_TASK_LIST
        try:
            result = self._get(url=url)
            u = result['user']
            li = result['level_info']

            username = u['username']
            userid = u['userid']
            coin = int(li['coin'])
            exp = int(li['exp'])
            level = int(li['level'])
            max_exp = int(li['max_exp'])
            sign_in_streak = int(
                result['task_list'][0]['tasks'][0]['sign_in_streak'])
            self.logger.debug(f'昵称[{username}] @{userid} [{level}级]')
            self.logger.debug(
                f'盒币[{coin}]|经验[{exp}/{max_exp}]|连续签到[{sign_in_streak}]天')
            return((username, userid, coin, (level, exp, max_exp), sign_in_streak))
        except (ClientException, KeyError, NameError) as e:
            self.logger.error(f'[*] 获取我的数据出错 [{e}]')
            return(('读取信息出错', 0, 0, (0, 0, 0), 0))

    def follow_user(self, userid: int, follow: bool = True) -> bool:
        '''
        关注/取关用户

        参数:
            userid: 用户id
            [follow]: True表示关注用户,False表示取关
        返回:
            bool: 是否成功
        '''
        if (userid == self.heybox_id):
            self.logger.warning('[*] 不能关注自己哦')
            return(False)
        url = URLS.FOLLOW_USER if follow else URLS.UNFOLLOW_USER
        data = {'following_id': userid}

        try:
            self._post(url=url, data=data)
            self.logger.debug('关注/取关用户成功')
            return(True)
        except ClientException as e:
            self.logger.error(f'[*] 关注/取关用户失败 [{e}]')
            return(False)

    def get_user_follows(self, userid: int = 0, amount: int = 30) -> list:
        '''
        获取用户关注列表,失败返回False

        参数:
            [userid]: 用户id,不填代入自己
            [amount]: 要拉取的数量
        返回:
            list: [(userid,username,relation),……],粉丝列表,relation释义参考static.RelationType
        '''
        def get(offset: int):
            params = {'userid': userid, 'offset': offset, 'limit': 30}
            result = self._get(url=url, params=params, key='follow_list')
            tmp = []
            for f in result:
                try:
                    uid = int(f['userid'])
                    uname = f['username']
                    relation = f['is_follow']
                    # 把自己跟自己的关系设为互关
                    if uid == self.heybox_id:
                        relation = RelationType.BOthFollowed
                    tmp.append((uid, uname, relation))
                except KeyError as e:
                    self.logger.debug(f'提取动态列表出错[{f}]')
            self.logger.debug(f'拉取[{len(tmp)}]个粉丝')
            return(tmp)
        # ==========================================
        url = URLS.GET_FOLLOW_LIST
        userid = userid or self.heybox_id
        followlist = []
        empty = 0
        error = 0
        for i in range(0, amount//30+2):
            try:
                self.logger.debug(f'拉取第[{i+1}]批关注列表')
                tmp = get(i * 30)
                if tmp:
                    followlist = ex_extend(followlist, tmp)
                else:
                    self.logger.debug('[*] 关注列表为空,可能遇到错误')
                    empty += 1
                    if empty > EMPTY_RETRYS:
                        self.logger.debug('[*] 空结果达到上限,停止操作')
                        break
                if len(followlist) >= amount:
                    break
            except ClientException as e:
                self.logger.debug(f'[*] 拉取关注列表出错 [{e}]')
                error += 1
                if error > ERROR_RETRYS:
                    self.logger.error('[*] 错误次数达到上限,停止操作')
                    break

        followlist = followlist[:amount]
        if len(followlist) > 0:
            self.logger.debug(f'操作完成,拉取了[{len(followlist)}]个关注')
        else:
            self.logger.debug('拉取完毕,关注列表为空,可能遇到错误')
        return(followlist)

    def get_user_fans(self, userid: int = 0, amount: int = 30) -> list:
        '''
        获取用户粉丝列表,失败返回False

        参数:
            [userid]: 用户id,不填代入自己
            [amount]: 要拉取的数量
        返回:
            list: [(userid,username,relation),……],粉丝列表,relation释义参考static.RelationType
        '''
        def get(offset: int):
            params = {'userid': userid, 'offset': offset, 'limit': 30}
            result = self._get(url=url, params=params, key='follow_list')
            tmp = []
            for f in result:
                try:
                    uid = int(f['userid'])
                    uname = f['username']
                    relation = f['is_follow']
                    # 把自己跟自己的关系设为互关
                    if uid == self.heybox_id:
                        relation = RelationType.BOthFollowed
                    tmp.append((uid, uname, relation))
                except KeyError as e:
                    self.logger.debug(f'提取动态列表出错[{f}]')
            self.logger.debug(f'拉取[{len(tmp)}]个粉丝')
            return(tmp)
        # ==========================================
        url = URLS.GET_FAN_LIST
        userid = userid or self.heybox_id
        fanlist = []
        empty = 0
        error = 0
        for i in range(0, amount//30+2):
            try:
                self.logger.debug(f'拉取第[{i+1}]批粉丝列表')
                tmp = get(i * 30)
                if tmp:
                    fanlist = ex_extend(fanlist, tmp)
                else:
                    self.logger.debug('[*] 粉丝列表为空,可能遇到错误')
                    empty += 1
                    if empty > EMPTY_RETRYS:
                        self.logger.debug('[*] 空结果达到上限,停止操作')
                        break
                if len(fanlist) >= amount:
                    break
            except ClientException as e:
                self.logger.debug(f'[*] 拉取粉丝列表出错 [{e}]')
                error += 1
                if error > ERROR_RETRYS:
                    self.logger.error('[*] 错误次数达到上限,停止操作')
                    break

        fanlist = fanlist[:amount]
        if len(fanlist) > 0:
            self.logger.debug(f'操作完成,拉取了[{len(fanlist)}]个粉丝')
        else:
            self.logger.debug('拉取完毕,粉丝列表为空,可能遇到错误')
        return(fanlist)

    def get_new_fans(self) -> list:
        '''
        获取新粉丝

        返回:
            list: 新粉丝列表
        '''
        fanlist = self.get_user_fans(self.heybox_id, 60)
        newfans = user_relation_filter(fanlist, RelationType.HeFollowedMe)
        return(newfans)

    def get_auth_info(self) -> (bool, int):
        '''
        获取自己的安全设置,失败返回False

        成功返回:
            bool:有密码?
            int:手机号
        失败返回:
            False
        '''
        url = URLS.GET_AUTH_INFO
        try:
            result = self._get(url)

            r = result[0]
            has_password = bool(r['has_password'] == 1)
            phone_num = r['src_id']
            if has_password:
                self.logger.debug(f'手机号[{phone_num}]')
            else:
                self.logger.debug(f'手机号[{phone_num}],**未设置密码**')
            return((has_password, phone_num))
        except ClientException as e:
            self.logger.error(f'获取安全信息出错[{e}]')
            return(False)



'''
# @Author       : Chr_
# @Date         : 2020-07-30 22:21:56
# @LastEditors  : Chr_
# @LastEditTime : 2020-08-20 17:47:51
# @Description  : 社区模块,负责[社区]TAB下的内容
'''

from .network import Network
from .static import URLS, EMPTY_RETRYS, ERROR_RETRYS,CommentType
from .utils import ex_extend
from .error import ClientException


class Community(Network):

    def __init__(self, account: dict, hbxcfg: dict, debug: bool):
        super().__init__(account, hbxcfg, debug)

    def debug(self):
        super().debug()
        # self.send_community_comment(43817628,'test')

    def get_recommend_post(self, amount: int = 8) -> list:
        '''
        获取[推荐]下的文章

        参数:
            [amount]: 需要拉取的数量
        成功返回:
            list: 文章id列表,[(linkid,title,desc,userid),……]
        '''
        def get(lastval: str, pull: int = 0) -> (list, str):
            params = {'pull': pull, 'use_history': 0, 'lastval': lastval}
            if not lastval:
                params.pop('lastval')
                params.pop('use_history')
            result = self._get(url=url, params=params)
            tmp = []
            val = result.get('lastval')
            for l in result['links']:
                title = l['title']
                desc = l['description']
                linkid = l['linkid']
                userid = l['user']['userid']
                tmp.append((linkid, title, desc, userid))
            self.logger.debug(f'拉取了{len(tmp)}条帖子')
            return((tmp, val))
        # ==========================================
        url = URLS.GET_RECOMMEND_POST
        postlist = []
        empty = 0
        error = 0
        lastval = None
        for i in range(0, amount//8 + 2):
            try:
                self.logger.debug(f'拉取第[{i+1}]批帖子')
                tmp, lastval = get(lastval, 1 if i == 0 else 0)
                if tmp:
                    postlist = ex_extend(postlist, tmp)
                else:
                    self.logger.debug('[*] 帖子列表为,可能遇到错误')
                    empty += 1
                    if empty > EMPTY_RETRYS:
                        self.logger.debug('[*] 空结果达到上限,停止操作')
                        break
                if len(postlist) >= amount:
                    break
            except ClientException as e:
                self.logger.debug(f'[*] 拉取推荐帖子列表出错 [{e}]')
                error += 1
                if error > ERROR_RETRYS:
                    self.logger.debug('[*] 错误次数达到上限,停止操作')
                    break
        postlist = postlist[:amount]
        if len(postlist) > 0:
            self.logger.debug(f'操作完成,拉取了[{len(postlist)}]条帖子')
        else:
            self.logger.warning('[*] 拉取完毕,帖子列表为空,请检查参数')
        return(postlist)

    def get_recommend_post_id(self, amount: int = 8) -> list:
        '''
        获取首页文章id列表

        参数:
            [amount]: 需要拉取的数量
        成功返回:
            list: 文章id列表,[linkid,……]
        '''
        postlist = self.get_recommend_post(amount)
        idlist = [x[0] for x in postlist]
        return(idlist)

    def send_community_comment(self, linkid: int, message: str) -> bool:
        '''
        发送社区评论

        参数:
            linkid: 文章id
            message: 文字评论内容
        返回:
            操作是否成功
        '''
        result = self._send_comment(
            linkid, message, CommentType.Community, 0)
        return(result)